/******************************** */
/* Rule Set Based Access Control  */
/* Author: Amon Ott               */
/* Helper functions for all parts */
/* Last modified: 02/11/97        */
/******************************** */

#include <rsbac/getname.h>
#include <rsbac/helpers.h>
#include <rsbac/error.h>
#include <string.h>

#ifndef NULL
#define NULL ((void *) 0)
#endif

static char  request_list[R_NONE+1][24] = {
                        "ADD_TO_KERNEL",
                        "ALTER",
                        "APPEND_OPEN",
                        "CHANGE_GROUP",
                        "CHANGE_OWNER",
                        "CHDIR",
                        "CLONE",
                        "CLOSE",
                        "CREATE",
                        "DELETE",
                        "EXECUTE",
                        "GET_PERMISSIONS_DATA",
                        "GET_STATUS_DATA",
                        "LINK_HARD",
                        "MODIFY_ACCESS_DATA",
                        "MODIFY_ATTRIBUTE",
                        "MODIFY_PERMISSIONS_DATA",
                        "MODIFY_SYSTEM_DATA",
                        "MOUNT",
                        "READ",
                        "READ_ATTRIBUTE",
                        "READ_WRITE_OPEN",
                        "READ_OPEN",
                        "REMOVE_FROM_KERNEL",
                        "RENAME",
                        "SEARCH",
                        "SEND_SIGNAL",
                        "SHUTDOWN",
                        "SWITCH_LOG",
                        "SWITCH_MODULE",
                        "TERMINATE",
                        "TRACE",
                        "TRUNCATE",
                        "UMOUNT",
                        "WRITE",
                        "WRITE_OPEN",
                        "NONE" };

static char  result_list[UNDEFINED+1][12] = {
                         "NOT_GRANTED",
                         "GRANTED",
                         "DO_NOT_CARE",
                         "UNDEFINED" };

static char  attribute_list[A_none+1][18] = {
                          "security_level",
                          "object_category",
                          "data_type",
                          "system_role",
                          "integrity_role",
                          "pm_role",
                          "process_type",
                          "pm_process_type",
                          "pm_current_task",
                          "object_type",
                          "pm_object_class",
                          "pm_ipc_purpose",
                          "pm_object_type",
                          "program_type",
                          "pm_program_type",
                          "pm_tp",
                          "pm_task_set",
                          "owner",
                          "pseudo",
                          "current_sec_level",
                          "min_write_open",
                          "max_read_open",
                          "mac_auto",
                          "mac_trusted",
                          "cwi_rel_id",
#ifdef __KERNEL__
                          /* adf-request helpers */
                          "group",
                          "sockaddr",
                          "signal",
                          "mode",
                          "nlink",
#endif
                          "none" };

static char  target_list[T_NONE+1][8] = {
                          "FILE",
                          "DIR",
                          "IPC",
                          "SCD",
                          "USER",
                          "PROCESS",
                          "NONE" };

static char  switch_target_list[PM+1][4] = {
                          "MAC",
                          "CWI",
                          "FC",
                          "SIM",
                          "PM" };

static char  error_list[RSBAC_EDECISIONMISMATCH][25] = {
                          "RSBAC_EPERM",
                          "RSBAC_EACCESS",
                          "RSBAC_EREADFAILED",
                          "RSBAC_EWRITEFAILED",
                          "RSBAC_EINVALIDPOINTER",
                          "RSBAC_ENOROOTDIR",
                          "RSBAC_EPATHTOOLONG",
                          "RSBAC_ENOROOTDEV",
                          "RSBAC_ENOTFOUND",
                          "RSBAC_ENOTINITIALIZED",
                          "RSBAC_EREINIT",
                          "RSBAC_ECOULDNOTADDDEVICE",
                          "RSBAC_ECOULDNOTADDITEM",
                          "RSBAC_ECOULDNOTCREATEPATH",
                          "RSBAC_EINVALIDATTR",
                          "RSBAC_EINVALIDDEV",
                          "RSBAC_EINVALIDTARGET",
                          "RSBAC_EINVALIDVALUE",
                          "RSBAC_EEXISTS",
                          "RSBAC_EINTERNONLY",
                          "RSBAC_EINVALIDREQUEST",
                          "RSBAC_EDECISIONMISMATCH"};

/* Attribute types */

#ifndef __KERNEL__
static char  attribute_param_list[A_none+1][117] = {
                          "\t0 = unclassified, 1 = confidential, 2 = secret,\n\t\t\t3 = top secret",
                          "0 = general, 1 = security, 2 = system",
                          "\t0 = none, 1 = CDI, 2 = CDIIC, 3 = SI",
                          "\t0 = user, 1 = security officer, 2 = administrator",
                          "\t0 = none, 1 = TP-user, 2 = TP-manager, 3 = IVP-user",
                          "\t0 = user, 1 = security officer,\n\t\t\t2 = data protection officer, 3 = TP-manager,\n\t\t\t4 = system-admin",
                          "\t0 = none, 1 = TP, 2 = IVP, 3 = TPICD",
                          "0 = none, 1 = TP",
                          "Task-ID (positive integer)",
                          "\t0 = file, 1 = dir, 2 = ipc",
                          "Class-ID (integer >= 2)",
                          "\tPurpose-ID (positive integer)",
                          "\t0 = none, 1 = TP, 2 = personal data,\n\t\t\t3 = non-personal data, 4 = ipc, 5 = dir",
                          "\t0 = none, 1 = TP, 2 = IVP, 3 = TPICD",
                          "0 = none, 1 = TP",
                          "\t\tTP-ID (positive integer)",
                          "\tpm-task-list-ID (positive integer)",
                          "\t\tuser-ID (non-negative integer)",
                          "\t\tuser-pseudo (positive long integer)",
                          "0 = unclassified, 1 = confidential, 2 = secret,\n\t\t\t3 = top secret",
                          "\t0 = unclassified, 1 = confidential, 2 = secret,\n\t\t\t3 = top secret",
                          "\t0 = unclassified, 1 = confidential, 2 = secret,\n\t\t\t3 = top secret",
                          "\t0 = false, 1 = true",
                          "\t0 = false, 1 = true",
                          "\tCWI relation-id (positive integer)",
                          "\t\tINVALID!" };
#endif

/*****************************************/

char * get_request_name(char * request_name,
                        enum  rsbac_adf_request_t request)
  {
    if(!request_name)
      return(NULL);
    strcpy(request_name, request_list[request]);
    return(request_name);
  };

enum   rsbac_adf_request_t get_request_nr(char * request_name)
  {
     enum  rsbac_adf_request_t i;
    
    if(!request_name)
      return(R_NONE);
    for (i = 0; i < R_NONE; i++)
      {
        if (!strcmp(request_name,request_list[i]))
          {
            return(i);
          }
      }
    return(R_NONE);
  };


char * get_result_name(char * res_name,
                       enum   rsbac_adf_req_ret_t res)
  {
    if(!res_name)
      return(NULL);
    strcpy(res_name, result_list[res]);
    return(res_name);
  };

enum   rsbac_adf_req_ret_t get_result_nr(char * res_name)
  {
     enum  rsbac_adf_req_ret_t i;
    
    if(!res_name)
      return(UNDEFINED);
    for (i = 0; i < UNDEFINED; i++)
      {
        if (!strcmp(res_name,result_list[i]))
          {
            return(i);
          }
      }
    return(UNDEFINED);
  };


char * get_attribute_name(     char *            attr_name,
                          enum rsbac_attribute_t attr)
  {
    if(!attr_name)
      return(NULL);
    strcpy(attr_name, attribute_list[attr]);
    return(attr_name);
  };

enum   rsbac_attribute_t get_attribute_nr(char * attr_name)
  {
     enum  rsbac_attribute_t i;
    
    if(!attr_name)
      return(A_none);
    for (i = 0; i < A_none; i++)
      {
        if (!strcmp(attr_name,attribute_list[i]))
          {
            return(i);
          }
      }
    return(A_none);
  };


char * get_target_name(char * target_type_name,
                       enum   rsbac_target_t target,
                       char * target_id_name,
                       union  rsbac_target_id_t tid)
  {
    char help_name[80];

    if(!target_type_name || !target_id_name)
      return(NULL);
    switch(target)
      {
#ifdef __KERNEL__
        case T_FILE:    strcpy (target_type_name, "FILE");
                        strcpy (target_id_name, "Device ");
                        ulongtostr(help_name, (u_long) tid.file.device);
                        strcat(target_id_name, help_name);
                        strcat(target_id_name, " Inode ");
                        ulongtostr(help_name, tid.file.inode);
                        strcat(target_id_name, help_name);
                        break;
        case T_DIR:     strcpy (target_type_name, "DIR");
                        strcpy (target_id_name, "Device ");
                        ulongtostr(help_name, (u_long) tid.dir.device);
                        strcat(target_id_name, help_name);
                        strcat(target_id_name, " Inode ");
                        ulongtostr(help_name, tid.dir.inode);
                        strcat(target_id_name, help_name);
                        break;
#endif
        case T_IPC:     strcpy (target_type_name, "IPC");
                        switch (tid.ipc.type)
                          {
                            case I_sem: strcpy(target_id_name, "Sem-ID ");
                                        break;
                            case I_msg: strcpy(target_id_name, "Msg-ID ");
                                        break;
                            case I_shm: strcpy(target_id_name, "Shm-ID ");
                                        break;
                            case I_socket: strcpy(target_id_name, "Sock-ID ");
                                        break;
                            default:    strcpy(target_id_name, "???-ID ");
                                        break;
                          };
                        inttostr(help_name, tid.ipc.id_nr);
                        strcat(target_id_name, help_name);
                        break;
        case T_SCD:     strcpy (target_type_name, "SCD");
                        inttostr(target_id_name, tid.scd);
                        break;
        case T_USER:    strcpy (target_type_name, "USER");
                        inttostr(target_id_name, tid.user);
                        break;
        case T_PROCESS: strcpy (target_type_name, "PROCESS");
                        ulongtostr(target_id_name, tid.process);
                        break;
        case T_NONE:    strcpy (target_type_name, "NONE");
                        strcpy (target_id_name, "NONE");
                        break;
        default:        strcpy (target_type_name, "ERROR!!!");
                        strcpy (target_id_name, "ERROR!!!");
      };
    return(target_type_name);
  };

enum   rsbac_target_t get_target_nr(char * target_name)
  {
     enum  rsbac_target_t i;
    
    if(!target_name)
      return(T_NONE);
    for (i = 0; i < T_NONE; i++)
      {
        if (!strcmp(target_name,target_list[i]))
          {
            return(i);
          }
      }
    return(T_NONE);
  };

char * get_switch_target_name(char * switch_name,
                        enum  rsbac_switch_target_t target)
  {
    if(!switch_name)
      return(NULL);
    strcpy(switch_name, switch_target_list[target]);
    return(switch_name);
  };

enum   rsbac_switch_target_t get_switch_target_nr(char * switch_name)
  {
     enum  rsbac_switch_target_t i;
    
    if(!switch_name)
      return(SW_NONE);
    for (i = 0; i < PM; i++)
      {
        if (!strcmp(switch_name,switch_target_list[i]))
          {
            return(i);
          }
      }
    return(SW_NONE);
  };

char * get_error_name(char * error_name,
                      int target)
  {
    if(!error_name || (target<RSBAC_EPERM) || (target>RSBAC_EDECISIONMISMATCH))
      return(NULL);
    strcpy(error_name, error_list[target-RSBAC_EPERM]);
    return(error_name);
  };

#ifndef __KERNEL__
char * get_attribute_param(     char *            attr_name,
                          enum rsbac_attribute_t attr)
  {
    if(!attr_name)
      return(NULL);
    strcpy(attr_name, attribute_param_list[attr]);
    return(attr_name);
  };
#endif
