/******************************** */
/* Rule Set Based Access Control  */
/* Author: Amon Ott               */
/* Helper functions for all parts */
/* Last modified: 18/08/97        */
/******************************** */

#include <rsbac/helpers.h>
#include <rsbac/getname.h>
#include <asm/segment.h>
#include <linux/kernel.h>

char * inttostr(char * str, int i)
  {
    int j = 0;
    
    if(!str)
      return(NULL);
      
    if (i<0)
      {
        str[j] = '-';
        j++;
        i = -i;
      }
    if (i>=10000)
      {
        str[j] = '0' + (i / 10000);
        j++;
      }
    if (i>=1000)
      {
        str[j] = '0' + ((i % 10000) / 1000);
        j++;
      }
    if (i>=100)
      {
        str[j] = '0' + ((i % 1000) / 100);
        j++;
      }
    if (i>=10)
      {
        str[j] = '0' + ((i % 100) / 10);
        j++;
      }
    str[j] = '0' + (i % 10);
    j++;
    str[j] = 0;
    return (str);
  };

char * ulongtostr(char * str, u_long i)
  {
    int    j = 0;
    u_long k = 1000000000;
    
    if(!str)
      return(NULL);

    if (i>=k)
      {
        str[j] = '0' + ((i / k) % 100);
        j++;
      }
    k /= 10;

    while (k>1)
      {
        if (i>=k)
          {
            str[j] = '0' + ((i % (k*10)) / k);
            j++;
          }
        k /= 10;
      };

    str[j] = '0' + (i % 10);
    j++;
    str[j] = 0;
    return (str);
  };

char * longtostr(char * str, long i)
  {
    int    j = 0;
    u_long k = 1000000000;
    
    if(!str)
      return(NULL);

    if (i<0)
      {
        str[0] = '-';
        j = 1;
        i = -i;
      }
    if (i>=k)
      {
        str[j] = '0' + ((i / k) % 100);
        j++;
      }
    k /= 10;

    while (k>1)
      {
        if (i>=k)
          {
            str[j] = '0' + ((i % (k*10)) / k);
            j++;
          }
        k /= 10;
      };

    str[j] = '0' + (i % 10);
    j++;
    str[j] = 0;
    return (str);
  };

#ifdef __KERNEL__
/****************************************************************/
/* Access to user data space                                    */

void rsbac_get_user(unsigned char * kern_p, unsigned char * user_p, int size)
  {
    int i;

    if(kern_p && user_p && (size > 0))
      {
        for (i = 0; i < size; i++)
          {
            *kern_p = get_user(user_p);
            kern_p++;
            user_p++;
          }
      }
  };

void rsbac_put_user(unsigned char * kern_p, unsigned char * user_p, int size)
  {
    int i;

    if(kern_p && user_p && (size > 0))
      {
        for (i = 0; i < size; i++)
          {
            put_user(*kern_p, user_p);
            kern_p++;
            user_p++;
          }
      }
  };

/**********************************************/
/*  shortcut for error calling rsbac_get_attr */

void rsbac_get_attr_error(char * func, enum rsbac_adf_request_t request)
  {
    char  name[80];
    
    if(func)
      {
        get_request_name(name, request);
        printk (KERN_WARNING
                "%s: rsbac_get_attr for %s returned error!", func, name);
      }
  };

#endif /* __KERNEL__ */
